
import { promises as fs } from 'fs';
import { parse as parse_yaml } from 'yaml';
import { resolve as resolve_path, dirname } from 'path';
import { MarkdownOptions } from '@doc-utils/markdown2html';

export async function read_config(file: string) {
	const path = resolve_path(process.cwd(), file);
	const yaml = await fs.readFile(path, 'utf8');
	const config = parse_yaml(yaml);
	validate_config(config);
	process_markdown_config(config);
	resolve_paths(path, config);
	return config;
}

export interface RSSConfig {
	dir_path?: string;
	out_file?: string;
	front_matter_field?: string;
}

export interface CalendarConfig {
	dir_path?: string;
	out_file?: string;
	front_matter_field?: string;
}

export interface Config {
	metadata: string;
	base_url: string;
	input: {
		root: string;
		raw?: string[];
		text?: string[];
		markdown?: string[];
		'schema+json'?: string[];
		'schema+yaml'?: string[];
		'openapi+json'?: string[];
		'openapi+yaml'?: string[];
		// ...
	};
	templates?: {
		layouts?: string;
		partials?: string;
		themes?: string;
		tags?: [ string, string ];
		env?: string[];
	};
	output: {
		root: string;
		include_inputs?: string[];
		include_yaml_and_json?: boolean;
		include_intermediate_markdown?: boolean;
	};
	sitemap?: false | {
		out_file?: string;
		front_matter_field?: string;
	};
	rss?: false | RSSConfig[];
	events?: false | {
		front_matter_field?: string;
	};
	calendars?: false | CalendarConfig[];
	markdown?: Omit<MarkdownOptions, 'base_url' | 'inline' | 'extensions'>;
	schema?: {
		// 
	};
	openapi?: {
		// ;
	}
	// ...
}

function validate_config(config: unknown) : asserts config is Config {
	// todo: validate config
}

function resolve_paths(file_path: string, config: Config) {
	const base_path = dirname(file_path);
	config.input.root = resolve_path(base_path, config.input.root);
	config.output.root = resolve_path(base_path, config.output.root);
	config.metadata = resolve_path(base_path, config.metadata);

	if (config.templates?.layouts) {
		config.templates.layouts = resolve_path(base_path, config.templates.layouts);
	}

	if (config.templates?.partials) {
		config.templates.partials = resolve_path(base_path, config.templates.partials);
	}

	if ((config.sitemap as boolean) === true) {
		config.sitemap = { };
	}

	if ((config.events as boolean) === true) {
		config.events = { };
	}

	if (Array.isArray(config.rss)) {
		for (const rss_conf of config.rss) {
			if (rss_conf.dir_path) {
				rss_conf.dir_path = resolve_path(config.input.root, rss_conf.dir_path);
				rss_conf.out_file = resolve_path(config.output.root, rss_conf.out_file);
			}
		}
	}

	if (config.sitemap && typeof config.sitemap === 'object' && config.sitemap.out_file) {
		config.sitemap.out_file = resolve_path(config.output.root, config.sitemap.out_file);
	}

	if (Array.isArray(config.calendars)) {
		for (const cal_conf of config.calendars) {
			if (cal_conf.dir_path) {
				cal_conf.dir_path = resolve_path(config.input.root, cal_conf.dir_path);
				cal_conf.out_file = resolve_path(config.output.root, cal_conf.out_file);
			}
		}
	}
}

function process_markdown_config(config: any) {
	if (config?.markdown?.custom_elements) {
		if (config.markdown.custom_elements.tag_names) {
			const tags = new Set<string>(config.markdown.custom_elements.tag_names);
			config.markdown.custom_elements.tagNameCheck = (tag_name) => tags.has(tag_name);
			delete config.markdown.custom_elements.tag_names;
		}

		if (config.markdown.custom_elements.attribute_names) {
			const attrs = new Set<string>(config.markdown.custom_elements.attribute_names);
			config.markdown.custom_elements.attributeNameCheck = (attr_name) => attrs.has(attr_name);
			delete config.markdown.custom_elements.attribute_names;
		}
	}
}
